@@ -37,7 +37,7 @@ class PrefilterError(Exception):
 
 
 # RegExp to identify potential function names
-re_fun_name = re.compile(r'[a-zA-Z_]([a-zA-Z0-9_.]*) *$')
+re_fun_name = re.compile(r'[^\W\d]([\w.]*) *$')
 
 # RegExp to exclude strings with this start from autocalling.  In
 # particular, all binary operators should be excluded, so that if foo is
@@ -120,7 +120,7 @@ class PrefilterManager(Configurable):
     def __init__(self, shell=None, **kwargs):
         super(PrefilterManager, self).__init__(shell=shell, **kwargs)
         self.shell = shell
-        self.init_transformers()
+        self._transformers = []
         self.init_handlers()
         self.init_checkers()
 
@@ -128,14 +128,6 @@ class PrefilterManager(Configurable):
     # API for managing transformers
     #-------------------------------------------------------------------------
 
-    def init_transformers(self):
-        """Create the default transformers."""
-        self._transformers = []
-        for transformer_cls in _default_transformers:
-            transformer_cls(
-                shell=self.shell, prefilter_manager=self, parent=self
-            )
-
     def sort_transformers(self):
         """Sort the transformers by priority.
 
@@ -507,7 +499,7 @@ class AutocallChecker(PrefilterChecker):
             return None
 
         oinfo = line_info.ofind(self.shell) # This can mutate state via getattr
-        if not oinfo['found']:
+        if not oinfo.found:
             return None
 
         ignored_funs = ['b', 'f', 'r', 'u', 'br', 'rb', 'fr', 'rf']
@@ -516,10 +508,12 @@ class AutocallChecker(PrefilterChecker):
         if ifun.lower() in ignored_funs and (line.startswith(ifun + "'") or line.startswith(ifun + '"')):
             return None
 
-        if callable(oinfo['obj']) \
-               and (not self.exclude_regexp.match(line_info.the_rest)) \
-               and self.function_name_regexp.match(line_info.ifun):
-            return self.prefilter_manager.get_handler_by_name('auto')
+        if (
+            callable(oinfo.obj)
+            and (not self.exclude_regexp.match(line_info.the_rest))
+            and self.function_name_regexp.match(line_info.ifun)
+        ):
+            return self.prefilter_manager.get_handler_by_name("auto")
         else:
             return None
 
@@ -609,7 +603,7 @@ class AutoHandler(PrefilterHandler):
         the_rest = line_info.the_rest
         esc     = line_info.esc
         continue_prompt = line_info.continue_prompt
-        obj = line_info.ofind(self.shell)['obj']
+        obj = line_info.ofind(self.shell).obj
 
         # This should only be active for single-line input!
         if continue_prompt:
@@ -687,9 +681,6 @@ class EmacsHandler(PrefilterHandler):
 #-----------------------------------------------------------------------------
 
 
-_default_transformers = [
-]
-
 _default_checkers = [
     EmacsChecker,
     MacroChecker,
